==> stereo-img-1.10.0.tgz: package/parsers/stereo-parser/stereo-parser.js <==
// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import exifr from 'exifr'

/**
 * fetch the image from URL
 * return left and right eye images from either left / right, right / left, top / bottom, bottom / top. 
 * @Param {string} url - image url
 * @Param {Object} (options) - Parsing options: type: 'left-right' (default), 'right-left' or 'top-bottom', angle: '180' or '360'
 * */
async function parseStereo(url, options) {
  const image = await createImageFromURL(url);
  //image.crossOrigin = "Anonymous";
  
  const canvas = document.createElement('canvas');
  const ctx = canvas.getContext('2d', {willReadFrequently: true});
  const width = image.width;
  const height = image.height;
  canvas.width = width;
  canvas.height = height;
  ctx.drawImage(image, 0, 0);

  const exif = await exifr.parse(url, {
    xmp: true,
    multiSegment: true
  })


  // Images
  let type = options.type || 'left-right';

  // Heuristics for Canon RF5.2mm F2.8 L DUAL FISHEYE
  // Unprocessed pictures are right-left. Pictures processed with EOS VR Utility software are left-right.
  if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE') {
    if(exif?.Software?.includes('EOS VR Utility')) {
      type = 'left-right';
    } else {
      type = 'right-left';
    }
  }

  let leftEye;
  let rightEye;

  switch(type) {
    case 'left-right':
      leftEye =  ctx.getImageData(0, 0, width / 2, height);
      rightEye = ctx.getImageData(width / 2, 0, width / 2, height);
      break;
    case 'right-left':
      leftEye =  ctx.getImageData(width / 2, 0, width / 2, height);
      rightEye = ctx.getImageData(0, 0, width / 2, height);
      break;
    case 'top-bottom':
      leftEye = ctx.getImageData(0, 0, width, height / 2); 
      rightEye = ctx.getImageData(0, height / 2, width, height / 2);
      break;
    case 'bottom-top':
      leftEye = ctx.getImageData(0, height / 2, width, height / 2);
      rightEye = ctx.getImageData(0, 0, width, height / 2);
      break;
  }
  
  let angleOfViewFocalLengthIn35mmFormat = function(focalLengthIn35mmFormat) {
    // https://en.wikipedia.org/wiki/Angle_of_view#Common_lens_angles_of_view
    // https://en.wikipedia.org/wiki/35_mm_equivalent_focal_length
    // angle of view on the diagonal (35mm is 24 mm (vertically) × 36 mm (horizontal), giving a diagonal of about 43.3 mm)
    const diagonalAngle = 2 * Math.atan(43.3 / (2 * focalLengthIn35mmFormat));
    // Pi / 4 for a square.
    const halfAngle = Math.atan(height / (width / 2));
    const horizontalAngle = diagonalAngle * Math.cos(halfAngle);
    const verticalAngle = diagonalAngle * Math.sin(halfAngle);

    return {diagonalAngle, horizontalAngle, verticalAngle};
  }


  // Angles
  let phiLength;
  let thetaLength;

  if(options?.angle === "180" || options?.angle === 180) {
    phiLength = Math.PI;
    thetaLength = Math.PI;

  } else if(options?.angle === "360" || options?.angle === 360) {
    phiLength = Math.PI * 2;
    thetaLength = Math.PI;

  } else if(exif?.FocalLengthIn35mmFormat) {
    const angle = angleOfViewFocalLengthIn35mmFormat(exif.FocalLengthIn35mmFormat);
    phiLength = angle.horizontalAngle;
    thetaLength = angle.verticalAngle;

  } else if(exif?.Make === 'GoPro' || url.includes('gopro') || url.includes('GOPR') ) {
    // GoPro (https://gopro.com/help/articles/question_answer/hero7-field-of-view-fov-information?sf96748270=1)
    phiLength = 2.1397737; // 122.6º
    thetaLength = 1.647591;  // 94.4º

  } else if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE') {
    // TODO: This lense has a 190º angle of view, but the viewer doesn't support any other angle than 180 for fisheye.
    phiLength = Math.PI;
    thetaLength = Math.PI;

  } else {
    const assumeFocalLengthIn35mmFormat = 27;
    const angle = angleOfViewFocalLengthIn35mmFormat(assumeFocalLengthIn35mmFormat);
    phiLength = angle.horizontalAngle;
    thetaLength = angle.verticalAngle;
  }

  const thetaStart = Math.PI / 2 - thetaLength / 2;


  // Projection
  let projection;
  if(options?.projection === 'fisheye') {
    projection = 'fisheye';
    
  } else if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE' && !exif?.Software?.includes('EOS VR Utility')) {
    // Assume pictures unprocessed with EOS VR Utility are fisheye
    projection = 'fisheye';
  }

  return {leftEye, rightEye, phiLength, thetaStart, thetaLength, projection};
}

async function createImageFromURL(url) {
  const image = new Image();
  image.src = url;
  return new Promise((resolve, reject) => {
    image.onload = () => {
      URL.revokeObjectURL(url);
      resolve(image);
    };
    image.onerror = reject;
  });
}

export {parseStereo}

==> stereo-img-1.11.0.tgz: package/parsers/stereo-parser/stereo-parser.js <==
// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import exifr from 'exifr'

/**
 * fetch the image from URL
 * return left and right eye images from either left / right, right / left, top / bottom, bottom / top. 
 * @Param {string} url - image url
 * @Param {Object} (options) - Parsing options: type: 'left-right' (default), 'right-left' or 'top-bottom', angle: '180' or '360'
 * */
async function parseStereo(url, options) {
  const image = await createImageFromURL(url);
  //image.crossOrigin = "Anonymous";
  
  const canvas = document.createElement('canvas');
  const ctx = canvas.getContext('2d', {willReadFrequently: true});
  const width = image.width;
  const height = image.height;
  canvas.width = width;
  canvas.height = height;
  ctx.drawImage(image, 0, 0);

  const exif = await exifr.parse(url, {
    xmp: true,
    multiSegment: true
  })


  // Images
  let type = options.type || 'left-right';

  // Heuristics for Canon RF5.2mm F2.8 L DUAL FISHEYE
  // Unprocessed pictures are right-left. Pictures processed with EOS VR Utility software are left-right.
  if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE') {
    if(exif?.Software?.includes('EOS VR Utility')) {
      type = 'left-right';
    } else {
      type = 'right-left';
    }
  }

  let leftEye;
  let rightEye;

  switch(type) {
    case 'left-right':
      leftEye =  ctx.getImageData(0, 0, width / 2, height);
      rightEye = ctx.getImageData(width / 2, 0, width / 2, height);
      break;
    case 'right-left':
      leftEye =  ctx.getImageData(width / 2, 0, width / 2, height);
      rightEye = ctx.getImageData(0, 0, width / 2, height);
      break;
    case 'top-bottom':
      leftEye = ctx.getImageData(0, 0, width, height / 2); 
      rightEye = ctx.getImageData(0, height / 2, width, height / 2);
      break;
    case 'bottom-top':
      leftEye = ctx.getImageData(0, height / 2, width, height / 2);
      rightEye = ctx.getImageData(0, 0, width, height / 2);
      break;
  }
  
  let angleOfViewFocalLengthIn35mmFormat = function(focalLengthIn35mmFormat) {
    // https://en.wikipedia.org/wiki/Angle_of_view#Common_lens_angles_of_view
    // https://en.wikipedia.org/wiki/35_mm_equivalent_focal_length
    // angle of view on the diagonal (35mm is 24 mm (vertically) × 36 mm (horizontal), giving a diagonal of about 43.3 mm)
    const diagonalAngle = 2 * Math.atan(43.3 / (2 * focalLengthIn35mmFormat));
    // Pi / 4 for a square.
    const halfAngle = Math.atan(height / (width / 2));
    const horizontalAngle = diagonalAngle * Math.cos(halfAngle);
    const verticalAngle = diagonalAngle * Math.sin(halfAngle);

    return {diagonalAngle, horizontalAngle, verticalAngle};
  }


  // Angles
  let phiLength;
  let thetaLength;

  if(options?.angle === "180" || options?.angle === 180) {
    phiLength = Math.PI;
    thetaLength = Math.PI;

  } else if(options?.angle === "360" || options?.angle === 360) {
    phiLength = Math.PI * 2;
    thetaLength = Math.PI;

  } else if(exif?.FocalLengthIn35mmFormat) {
    const angle = angleOfViewFocalLengthIn35mmFormat(exif.FocalLengthIn35mmFormat);
    phiLength = angle.horizontalAngle;
    thetaLength = angle.verticalAngle;

  } else if(exif?.Make === 'GoPro' || url.startsWith('gopro') || url.startsWith('GOPR') ) {
    // GoPro (https://gopro.com/help/articles/question_answer/hero7-field-of-view-fov-information?sf96748270=1)
    phiLength = 2.1397737; // 122.6º
    thetaLength = 1.647591;  // 94.4º

  } else if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE') {
    // TODO: This lense has a 190º angle of view, but the viewer doesn't support any other angle than 180 for fisheye.
    phiLength = Math.PI;
    thetaLength = Math.PI;

  } else {
    const assumeFocalLengthIn35mmFormat = 27;
    const angle = angleOfViewFocalLengthIn35mmFormat(assumeFocalLengthIn35mmFormat);
    phiLength = angle.horizontalAngle;
    thetaLength = angle.verticalAngle;
  }

  const thetaStart = Math.PI / 2 - thetaLength / 2;


  // Projection
  let projection;
  if(options?.projection === 'fisheye') {
    projection = 'fisheye';
    
  } else if(exif?.Make === 'Canon' && exif?.LensModel === 'RF5.2mm F2.8 L DUAL FISHEYE' && !exif?.Software?.includes('EOS VR Utility')) {
    // Assume pictures unprocessed with EOS VR Utility are fisheye
    projection = 'fisheye';
  }

  return {leftEye, rightEye, phiLength, thetaStart, thetaLength, projection};
}

async function createImageFromURL(url) {
  const image = new Image();
  image.src = url;
  return new Promise((resolve, reject) => {
    image.onload = () => {
      URL.revokeObjectURL(url);
      resolve(image);
    };
    image.onerror = reject;
  });
}

export {parseStereo}
